<?php

namespace BayCMS\Field;

abstract class Field extends \BayCMS\Base\BayCMSBase
{

    /**
     * Settings array
     * @var array
     */
    protected bool $error = false;
    protected string $inline_error='';

    protected string $name;
    protected mixed $value = '';



   /**
     * Create a single line text input 
     * @param \BayCMS\Base\BayCMSContext $context
     * @param array $settings
     * Keys:
     * 
     * @throws \BayCMS\Exception\missingData
     */
    public function __construct(\BayCMS\Base\BayCMSContext $context, array $settings)
    {
        $this->context = $context;
        if (!isset($settings['name'])) {
            throw new \BayCMS\Exception\missingData('Cannot create a field without name');
        }
        if (!isset($settings['description']))
            $settings['description'] = $settings['name'];
        $this->name = strtolower($settings['name']);
        if (isset($settings['default_value']))
            $this->value = $settings['default_value'];
        $this->settings = $settings;
    }

    /**
     * Returns the value of the field.
     * Some fields may return an array
     * @return mixed
     */
    public function getValue()
    {
        return $this->value;
    }

    /**
     * Returns the value of the field to display in the table
     * @return string
     */
    public function getDisplayValue()
    {
        return htmlspecialchars($this->value);
    }

    /**
     * Returns the name of the field
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Returns the id of the field. 
     * Per default this is FormName_FieldName
     * @param \BayCMS\Fieldset\Form $form
     * @return string
     */
    public function getID(\BayCMS\Fieldset\Form $form): string
    {
        return $this->settings['id'] ?? ($form->getName() . '_' . $this->name);
    }

    public function setID($id){
        $this->settings['id']=$id;
    }

    /**
     * Returns a SQL string used in table
     * @param mixed $prefix
     * @return mixed
     */
    public function getSQL($prefix = '')
    {
        return $this->settings['sql'] ?? ($prefix . '' . $this->name);
    }

    /**
     * Returns HTML-Code for the help
     * @param mixed $form
     * @return string
     */
    public function getHelp($form)
    {
        if (!isset($this->settings['help']))
            return '';

        $description = htmlspecialchars($this->getDescription(false) ?? '');
        $help_text = preg_replace('/\s\s+/', ' ', htmlspecialchars($this->settings['help']));
        $help_id = 'help_' . $this->getID($form);
        $help = ($GLOBALS['TE']->isBootstrap() ?
            '<a id="' . $help_id . '" data-toggle="popover" title="' . $description . '"
                data-content="' . $help_text . '"> <span class="glyphicon glyphicon-info-sign"></span></a>' :
            '<a id="' . $help_id . '" 
               onClick="showHelp(\'' . $description . "','" . $help_text . "','" . $help_id . "');\"> [?]</a>") .
            "<script>\$('a#" . $help_id . "').on('click',function(e){e.preventDefault(); return true;});</script>";
        return $help;
    }

    /**
     * Simple function to check if a field is
     * added to the main query
     * @return bool
     */
    public function noAddToQuery(): bool
    {
        return $this->settings['no_add_to_query'] ?? false;
    }

    /**
     * Set the value of a field
     * @param mixed $value
     * @return bool
     */
    public function setValue($value): bool
    {
        if ($value === null)
            $value = '';
        $this->value = trim($value);
        $this->error = false;
        if (($this->settings['non_empty'] ?? false) && !$this->value)
            $this->error = true;
        return $this->error;
    }

    /**
     * Set the error
     * @param mixed $error
     * @return void
     */
    protected function setError($error){
        $this->error=$error;
    }

    /**
     * Typically called with $_POST or database array
     * in the form-object
     * @param array $v
     * @param int $id
     * @return bool
     */
    public function setValueFromArray(array &$v, int $id=null): bool
    {
        if(! isset($v[$this->name]) && $this->get('setvalue_sql') && $id){
            $res=pg_query_params($this->context->getDbConn(),$this->get('setvalue_sql'),[$id]);
            if(pg_num_rows($res)) [$v[$this->name]]=pg_fetch_row($res,0);
        }
        return $this->setValue($v[$this->name] ?? null);
    }

    /**
     * Returns the description of a field
     * (with or without footnote)
     * @param bool $with_footnote
     * @return string
     */
    public function getDescription(bool $with_footnote = true)
    {
        $out = $this->settings['description'];
        if ($with_footnote) {
            if ($this->settings['footnote'] ?? false)
                $out .= " " . $this->settings['footnote'][0];
            else if ($this->settings['non_empty'] ?? false)
                $out .= " *";

            if ($this->error)
                $out = '<span style="color:#ff0000">' . $out . "</span>";

        }
        return $out;
    }

    /**
     * Creates a HTML table row
     * @return string
     */
    public function getTableRow()
    {
        if ($this->settings['not_in_table'] ?? false)
            return '';
        $out = "<tr><td>" . $this->settings['description'] . "</td><td>"
            . $this->getDisplayValue() . "</td></tr>\n";
        return $out;
    }

    /**
     * Creates the input 
     * @param \BayCMS\Fieldset\Form $form
     * @return string
     */
    abstract public function getInput(\BayCMS\Fieldset\Form $form):string;

    /**
     * Create the form row
     * @param \BayCMS\Fieldset\Form $form
     * @return string
     */
    public function getFormRow(\BayCMS\Fieldset\Form $form): string
    {
        $out = '<div class="' . $GLOBALS['TE']->getCSSClass('form_div') . '">';
        $out .= '<label for="' . $this->getID($form) . '"';
        if ($this->settings['label_css'] ?? '')
            $out .= ' style="' . $this->settings['label_css'] . '"';
        $out .= '>' . $this->getDescription() . $this->getHelp($form) . '</label>';
        $out .= $this->getInput($form);
        if ($this->inline_error)
            $out .= "\n<br/><span class=\"form_inlineerror\">" . $this->inline_error . "</span>";
        $out .= "</div>\n";
        return $out;
    }

    /**
     * Returns a array of footnotes in the form
     * ['**','Footnote']
     * @return mixed
     */
    public function getFootnote()
    {
        if (isset($this->settings["footnote"]))
            return $this->settings["footnote"];
        if (isset($this->settings["non_empty"]))
            return ['*', $this->t('mandatory','Pflichtfeld')];
        return false;
    }

    /**
     * Callback function to save the field data
     * to the database
     * @param \BayCMS\Fieldset\Form $form
     * @return bool
     */
    public function save(\BayCMS\Fieldset\Form $form): bool
    {
        return true;
    }

}